@@ -115,7 +115,7 @@ build_summary (EBookBackendFilePrivate *bfpriv)
 	db_error = db->cursor (db, NULL, &dbc, 0);
 
 	if (db_error != 0) {
-		g_warning ("build_summary: error building list\n");
+		g_warning (G_STRLOC ": db->cursor failed with %d", db_error);
 		return;
 	}
 
@@ -201,7 +201,7 @@ do_create(EBookBackendFile  *bf,
 			g_warning ("db->sync failed with %d", db_error);
 	}
 	else {
-		g_warning ("db->put failed with %d", db_error);
+		g_warning (G_STRLOC ": db->put failed with %d", db_error);
 		g_object_unref (contact);
 		contact = NULL;
 	}
@@ -254,6 +254,7 @@ e_book_backend_file_remove_contacts (EBookBackendSync *backend,
 
 		db_error = db->del (db, NULL, &id_dbt, 0);
 		if (0 != db_error) {
+			g_warning (G_STRLOC ": db->del failed with %d", db_error);
 			rv = GNOME_Evolution_Addressbook_ContactNotFound;
 			continue;
 		}
@@ -265,7 +266,7 @@ e_book_backend_file_remove_contacts (EBookBackendSync *backend,
 	if (removed_cards) {
 		db_error = db->sync (db, 0);
 		if (db_error != 0)
-			g_warning ("db->sync failed.\n");
+			g_warning (G_STRLOC ": db->sync failed with %d", db_error);
 	}
 
 	*ids = removed_cards;
@@ -315,10 +316,11 @@ e_book_backend_file_modify_contact (EBookBackendSync *backend,
 
 	/* get the old ecard - the one that's presently in the db */
 	db_error = db->get (db, NULL, &id_dbt, &vcard_dbt, 0);
-	if (0 != db_error)
+	if (0 != db_error) {
+		g_warning (G_STRLOC ": db->get failed with %d", db_error);
 		return GNOME_Evolution_Addressbook_ContactNotFound;
-
-	free (vcard_dbt.data);
+	}
+	g_free (vcard_dbt.data);
 
 	/* update the revisio (modified time of contact) */
 	set_revision (*contact);
@@ -331,7 +333,7 @@ e_book_backend_file_modify_contact (EBookBackendSync *backend,
 	if (0 == db_error) {
 		db_error = db->sync (db, 0);
 		if (db_error != 0)
-			g_warning ("db->sync failed.\n");
+			g_warning (G_STRLOC ": db->sync failed with %d", db_error);
 
 		e_book_backend_summary_remove_contact (bf->priv->summary, id);
 		e_book_backend_summary_add_contact (bf->priv->summary, *contact);
@@ -357,7 +359,7 @@ e_book_backend_file_get_contact (EBookBackendSync *backend,
 	DBT             id_dbt, vcard_dbt;
 	int             db_error = 0;
 
-	bf = E_BOOK_BACKEND_FILE (e_data_book_get_backend (book));
+	bf = E_BOOK_BACKEND_FILE (backend);
 	db = bf->priv->file_db;
 
 	string_to_dbt (id, &id_dbt);
@@ -367,11 +369,10 @@ e_book_backend_file_get_contact (EBookBackendSync *backend,
 	db_error = db->get (db, NULL, &id_dbt, &vcard_dbt, 0);
 
 	if (db_error == 0) {
-		*vcard = g_strdup (vcard_dbt.data);
-		printf ("backend %s\n", *vcard);
-		free (vcard_dbt.data);
+		*vcard = vcard_dbt.data;
 		return GNOME_Evolution_Addressbook_Success;
 	} else {
+		g_warning (G_STRLOC ": db->get failed with %d", db_error);
 		*vcard = g_strdup ("");
 		return GNOME_Evolution_Addressbook_ContactNotFound;
 	}
@@ -410,15 +411,15 @@ e_book_backend_file_get_contact_list (EBookBackendSync *backend,
 			vcard_dbt.flags = DB_DBT_MALLOC;
 
 			db_error = db->get (db, NULL, &id_dbt, &vcard_dbt, 0);
-			if (db_error == 0)
-				contact_list = g_list_append (contact_list, g_strdup (vcard_dbt.data));
-			else { 
+			if (db_error == 0) {
+				contact_list = g_list_append (contact_list, vcard_dbt.data);
+			} else {
+				g_warning (G_STRLOC ": db->get failed with %d", db_error);
 				status = GNOME_Evolution_Addressbook_OtherError ;
 				break;
 			}
 		}
 		g_ptr_array_free (ids, TRUE);
-		
 	} else {
 		search_needed = TRUE;
 		if (!strcmp (search, "(contains \"x-evolution-any-field\" \"\")"))
@@ -433,11 +434,13 @@ e_book_backend_file_get_contact_list (EBookBackendSync *backend,
 		db_error = db->cursor (db, NULL, &dbc, 0);
 
 		if (db_error != 0) {
+			g_warning (G_STRLOC ": db->cursor failed with %d", db_error);
 			/* XXX this needs to be some CouldNotOpen error */
 			return GNOME_Evolution_Addressbook_ContactNotFound;
 		}
 
 		memset (&vcard_dbt, 0, sizeof (vcard_dbt));
+		vcard_dbt.flags = DB_DBT_MALLOC;
 		memset (&id_dbt, 0, sizeof (id_dbt));
 		db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_FIRST);
 
@@ -448,7 +451,7 @@ e_book_backend_file_get_contact_list (EBookBackendSync *backend,
 			    || strcmp (id_dbt.data, E_BOOK_BACKEND_FILE_VERSION_NAME)) {
 
 				if ((!search_needed) || (card_sexp != NULL && e_book_backend_sexp_match_vcard  (card_sexp, vcard_dbt.data))) {
-					contact_list = g_list_append (contact_list, g_strdup (vcard_dbt.data));
+					contact_list = g_list_append (contact_list, vcard_dbt.data);
 				}
 			}
 
@@ -457,13 +460,16 @@ e_book_backend_file_get_contact_list (EBookBackendSync *backend,
 		}
 		g_object_unref (card_sexp);
 		
-		status = db_error != DB_NOTFOUND
-			? GNOME_Evolution_Addressbook_OtherError
-			: GNOME_Evolution_Addressbook_Success;
+		if (db_error == DB_NOTFOUND) {
+			status = GNOME_Evolution_Addressbook_Success;
+		} else {
+			g_warning (G_STRLOC ": dbc->c_get failed with %d", db_error);
+			status = GNOME_Evolution_Addressbook_OtherError;
+		}
 
 		db_error = dbc->c_close(dbc);
 		if (db_error != 0) {
-			g_warning("Could not close cursor: %d", db_error);
+			g_warning (G_STRLOC ": dbc->c_close failed with %d", db_error);
 		}
 	}
 
@@ -521,7 +527,7 @@ book_view_thread (gpointer data)
 	DB  *db;
 	DBT id_dbt, vcard_dbt;
 	int db_error;
-	gboolean stopped = FALSE;
+	gboolean stopped = FALSE, allcontacts;
 
 	d(printf ("starting initial population of book view\n"));
 
@@ -532,10 +538,13 @@ book_view_thread (gpointer data)
 	db = bf->priv->file_db;
 	query = e_data_book_view_get_card_query (book_view);
 
-	if ( ! strcmp (query, "(contains \"x-evolution-any-field\" \"\")"))
+	if ( ! strcmp (query, "(contains \"x-evolution-any-field\" \"\")")) {
 		e_data_book_view_notify_status_message (book_view, _("Loading..."));
-	else
+		allcontacts = TRUE;
+	} else {
 		e_data_book_view_notify_status_message (book_view, _("Searching..."));
+		allcontacts = FALSE;
+	}
 
 	d(printf ("signalling parent thread\n"));
 	g_mutex_lock (closure->mutex);
@@ -563,11 +572,10 @@ book_view_thread (gpointer data)
 			db_error = db->get (db, NULL, &id_dbt, &vcard_dbt, 0);
 
 			if (db_error == 0) {
-				/* notify_update will check if it matches for us */
-				e_data_book_view_notify_update_vcard (book_view, vcard_dbt.data);
+				e_data_book_view_notify_update_prefiltered_vcard (book_view, id, vcard_dbt.data);
 			}
 			else {
-				g_warning ("db->get returned %d", db_error);
+				g_warning (G_STRLOC ": db->get failed with %d", db_error);
 			}
 		}
 
@@ -596,8 +604,12 @@ book_view_thread (gpointer data)
 
 				/* don't include the version in the list of cards */
 				if (strcmp (id_dbt.data, E_BOOK_BACKEND_FILE_VERSION_NAME)) {
-					/* notify_update will check if it matches for us */
-					e_data_book_view_notify_update_vcard (book_view, vcard_dbt.data);
+					if (allcontacts)
+						e_data_book_view_notify_update_prefiltered_vcard (book_view, id_dbt.data, vcard_dbt.data);
+					else
+						e_data_book_view_notify_update_vcard (book_view, vcard_dbt.data);
+				} else {
+					g_free (vcard_dbt.data);
 				}
 
 				db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_NEXT);
@@ -705,9 +717,8 @@ e_book_backend_file_changes_foreach_key (const char *key, gpointer user_data)
 						vcard_string);
 
 		g_object_unref (contact);
-		
-		free (vcard_dbt.data);
 	}
+	g_free (vcard_dbt.data);
 }
 
 static EBookBackendSyncStatus
@@ -744,7 +755,7 @@ e_book_backend_file_get_changes (EBookBackendSync *backend,
 	db_error = db->cursor (db, NULL, &dbc, 0);
 
 	if (db_error != 0) {
-		g_warning ("e_book_backend_file_changes: error building list\n");
+		g_warning (G_STRLOC ": db->cursor failed with %d", db_error);
 	} else {
 		db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_FIRST);
 
@@ -772,6 +783,7 @@ e_book_backend_file_get_changes (EBookBackendSync *backend,
 				/* check what type of change has occurred, if any */
 				switch (e_dbhash_compare (ehash, id, vcard_string)) {
 				case E_DBHASH_STATUS_SAME:
+					g_free(vcard_string);
 					break;
 				case E_DBHASH_STATUS_NOT_FOUND:
 					ctx.add_cards = g_list_append (ctx.add_cards, vcard_string);
@@ -930,7 +942,7 @@ e_book_backend_file_upgrade_db (EBookBackendFile *bf, char *old_version)
 
 		db_error = db->cursor (db, NULL, &dbc, 0);
 		if (db_error != 0) {
-			g_warning ("unable to get cursor");
+			g_warning (G_STRLOC ": db->cursor failed with %d", db_error);
 			return FALSE;
 		}
 
@@ -1007,8 +1019,7 @@ e_book_backend_file_maybe_upgrade_db (EBookBackendFile *bf)
 	db_error = db->get (db, NULL, &version_name_dbt, &version_dbt, 0);
 	if (db_error == 0) {
 		/* success */
-		version = g_strdup (version_dbt.data);
-		free (version_dbt.data);
+		version = version_dbt.data;
 	}
 	else {
 		/* key was not in file */
@@ -1028,7 +1039,11 @@ e_book_backend_file_maybe_upgrade_db (EBookBackendFile *bf)
 #endif
 
 static void
+#if DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 3
+file_errcall (const DB_ENV *env, const char *buf1, const char *buf2)
+#else
 file_errcall (const char *buf1, char *buf2)
+#endif
 {
 	g_warning ("libdb error: %s", buf2);
 }
@@ -1077,6 +1092,11 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 			return GNOME_Evolution_Addressbook_OtherError;
 		}
 
+		/* Set the allocation routines to the non-aborting GLib functions */
+		env->set_alloc (env, (void *(*)(size_t))g_try_malloc, 
+				(void *(*)(void *, size_t))g_try_realloc,
+				g_free);
+
 		db_error = env->open (env, NULL, DB_CREATE | DB_INIT_MPOOL | DB_PRIVATE | DB_THREAD, 0);
 		if (db_error != 0) {
 			env->close(env, 0);
