@@ -340,7 +340,46 @@ get_attendee_prop (icalcomponent *icalcomp, const char *attendee)
 static void
 set_attendees_to_item (EGwItem *item, ECalComponent *comp, icaltimezone *default_zone, gboolean delegate, const char *user_email)
 {
-	if (e_cal_component_has_attendees (comp)) {
+	if (e_cal_component_get_vtype (comp) == E_CAL_COMPONENT_JOURNAL) {
+		if  (e_cal_component_has_organizer (comp)) {
+			icalcomponent *icalcomp = e_cal_component_get_icalcomponent (comp);
+			icalproperty *icalprop;
+			GSList *recipient_list = NULL;
+			const char *attendees = NULL;
+			char **emails, **iter;
+
+			for (icalprop = icalcomponent_get_first_property (icalcomp, ICAL_X_PROPERTY); icalprop; 
+					icalprop = icalcomponent_get_next_property (icalcomp, ICAL_X_PROPERTY)) {
+				if (g_str_equal (icalproperty_get_x_name (icalprop), "X-EVOLUTION-RECIPIENTS")) {
+					break;
+				}
+			}
+
+			if (icalprop)	 {
+				attendees = icalproperty_get_x (icalprop);
+				emails = g_strsplit (attendees, ";", -1);
+
+				iter = emails;
+				while (*iter) {
+					EGwItemRecipient *recipient;
+
+					recipient = g_new0 (EGwItemRecipient, 1);
+
+					recipient->email = g_strdup (*iter);
+					recipient->type = E_GW_ITEM_RECIPIENT_TO;
+
+					recipient_list = g_slist_prepend (recipient_list, recipient);
+					iter++;
+				}
+
+				e_gw_item_set_recipient_list (item, recipient_list);
+
+				g_strfreev (emails);
+				icalcomponent_remove_property (icalcomp, icalprop);
+				icalproperty_free (icalprop);
+			}
+		}
+	} else if (e_cal_component_has_attendees (comp)) {
 		GSList *attendee_list, *recipient_list = NULL, *al;
 
 		e_cal_component_get_attendee_list (comp, &attendee_list);	
@@ -384,12 +423,13 @@ set_attendees_to_item (EGwItem *item, ECalComponent *comp, icaltimezone *default
 	
 		/* recipient_list shouldn't be freed. Look into the function below. */
 		e_gw_item_set_recipient_list (item, recipient_list);
-
+	}
+	
+	if (e_cal_component_has_organizer (comp)) {
 		/* Send Options */
 		add_send_options_data_to_item (item, comp, default_zone);
-
 	}
-
+		
 	if (!delegate && e_cal_component_has_organizer (comp)) {
 		ECalComponentOrganizer cal_organizer;
 		EGwItemOrganizer *organizer = NULL;
@@ -579,7 +619,9 @@ set_properties_from_cal_component (EGwItem *item, ECalComponent *comp, ECalBacke
 			e_gw_item_set_completed (item, FALSE);
 
 		break;
-
+	case E_CAL_COMPONENT_JOURNAL:
+		e_gw_item_set_item_type (item, E_GW_ITEM_TYPE_NOTE);
+		break;
 	default :
 		g_object_unref (item);
 		return NULL;
@@ -631,7 +673,7 @@ set_properties_from_cal_component (EGwItem *item, ECalComponent *comp, ECalBacke
 	}
 	
 	/* all day event */
-	if (dt.value->is_date && e_gw_item_get_item_type (item) == E_GW_ITEM_TYPE_APPOINTMENT)
+	if (dt.value && dt.value->is_date && e_gw_item_get_item_type (item) == E_GW_ITEM_TYPE_APPOINTMENT)
 		e_gw_item_set_is_allday_event (item, TRUE);
 	
 	/* creation date */
@@ -690,7 +732,7 @@ set_properties_from_cal_component (EGwItem *item, ECalComponent *comp, ECalBacke
 	if (e_cal_component_has_attachments (comp)) {
 		e_cal_backend_groupwise_set_attachments_from_comp (comp, item); 
 	}
-
+	
 	return item;
 }
 
@@ -703,7 +745,6 @@ e_gw_item_new_from_cal_component (const char *container, ECalBackendGroupwise *c
 
 	item = e_gw_item_new_empty ();
 	e_gw_item_set_container_id (item, container);
-	
 	return set_properties_from_cal_component (item, comp, cbgw);
 }
 
@@ -803,6 +844,72 @@ set_attachments_to_cal_component (EGwItem *item, ECalComponent *comp, ECalBacken
 
 }
 
+static void
+set_default_alarms (ECalComponent *comp)
+{
+
+	GConfClient *gconf = gconf_client_get_default ();
+
+	if (gconf_client_get_bool (gconf, CALENDAR_CONFIG_DEFAULT_REMINDER, NULL)) {
+
+			ECalComponentAlarm *acomp;
+			int interval;
+			char * units;
+			enum {
+				DAYS,
+				HOURS,
+				MINUTES
+			} duration;	
+			ECalComponentAlarmTrigger trigger;
+
+			interval = gconf_client_get_int (gconf, CALENDAR_CONFIG_DEFAULT_REMINDER_INTERVAL, NULL);
+			units = gconf_client_get_string (gconf, CALENDAR_CONFIG_DEFAULT_REMINDER_UNITS, NULL);
+
+			if (units == NULL) 
+				duration = MINUTES;
+			else {
+				if (!strcmp (units, "days"))
+					duration = DAYS;
+				else if (!strcmp (units, "hours"))
+					duration = HOURS;
+				else
+					duration = MINUTES;
+				g_free (units);
+			}
+			acomp = e_cal_component_alarm_new ();
+
+			e_cal_component_alarm_set_action (acomp, E_CAL_COMPONENT_ALARM_DISPLAY);
+
+			trigger.type = E_CAL_COMPONENT_ALARM_TRIGGER_RELATIVE_START;
+			memset (&trigger.u.rel_duration, 0, sizeof (trigger.u.rel_duration));
+
+			trigger.u.rel_duration.is_neg = TRUE;
+
+			switch (duration) {
+			case MINUTES:
+				trigger.u.rel_duration.minutes = interval;
+				break;
+			case HOURS:	
+				trigger.u.rel_duration.hours = interval;
+				break;
+			case DAYS:	
+				trigger.u.rel_duration.days = interval;
+				break;
+			default:
+				e_cal_component_alarm_free (acomp);
+				g_object_unref (gconf);
+				return;
+			}
+
+			e_cal_component_alarm_set_trigger (acomp, trigger);
+			e_cal_component_add_alarm (comp, acomp);
+
+			e_cal_component_alarm_free (acomp);
+		}
+		g_object_unref (gconf);
+}
+
+
 ECalComponent *
 e_gw_item_to_cal_component (EGwItem *item, ECalBackendGroupwise *cbgw)
 {
@@ -838,6 +945,8 @@ e_gw_item_to_cal_component (EGwItem *item, ECalBackendGroupwise *cbgw)
 		e_cal_component_set_new_vtype (comp, E_CAL_COMPONENT_EVENT);
 	else if (item_type == E_GW_ITEM_TYPE_TASK)
 		e_cal_component_set_new_vtype (comp, E_CAL_COMPONENT_TODO);
+	else if (item_type == E_GW_ITEM_TYPE_NOTE)
+		e_cal_component_set_new_vtype (comp, E_CAL_COMPONENT_JOURNAL);
 	else {
 		g_object_unref (comp);
 		return NULL;
@@ -940,7 +1049,7 @@ e_gw_item_to_cal_component (EGwItem *item, ECalBackendGroupwise *cbgw)
 	if (t) {
 		itt_utc = icaltime_from_string (t);
 		
-		if (!is_allday) {
+		if (!is_allday && (item_type != E_GW_ITEM_TYPE_NOTE)) {
 			if (!icaltime_get_timezone (itt_utc))
 				icaltime_set_timezone (&itt_utc, icaltimezone_get_utc_timezone());
 			if (default_zone) {
@@ -959,8 +1068,7 @@ e_gw_item_to_cal_component (EGwItem *item, ECalBackendGroupwise *cbgw)
 		}	
 
 		e_cal_component_set_dtstart (comp, &dt);
-	}
-	else 
+	} else 
 		return NULL;
 	
 	/* UID */
@@ -1004,42 +1112,44 @@ e_gw_item_to_cal_component (EGwItem *item, ECalBackendGroupwise *cbgw)
 	} else
 		e_cal_component_set_classification (comp, E_CAL_COMPONENT_CLASS_NONE);
 
-	recipient_list = e_gw_item_get_recipient_list (item);
-	if (recipient_list != NULL) {
-		for (rl = recipient_list; rl != NULL; rl = rl->next) {
-			EGwItemRecipient *recipient = (EGwItemRecipient *) rl->data;
-			ECalComponentAttendee *attendee = g_new0 (ECalComponentAttendee, 1);
+	if (item_type != E_GW_ITEM_TYPE_NOTE) {
+		recipient_list = e_gw_item_get_recipient_list (item);
+		if (recipient_list != NULL) {
+			for (rl = recipient_list; rl != NULL; rl = rl->next) {
+				EGwItemRecipient *recipient = (EGwItemRecipient *) rl->data;
+				ECalComponentAttendee *attendee = g_new0 (ECalComponentAttendee, 1);
 
-			attendee->cn = g_strdup (recipient->display_name);
-			attendee->value = g_strconcat("MAILTO:", recipient->email, NULL);
-			if (recipient->type == E_GW_ITEM_RECIPIENT_TO)
-				attendee->role = ICAL_ROLE_REQPARTICIPANT;
-			else if (recipient->type == E_GW_ITEM_RECIPIENT_CC || recipient->type == E_GW_ITEM_RECIPIENT_BC)
-				attendee->role = ICAL_ROLE_OPTPARTICIPANT;
-			else 
-				attendee->role = ICAL_ROLE_NONE;
-			/* FIXME  needs a server fix on the interface 
-			 * for getting cutype and the status */
-			attendee->cutype = ICAL_CUTYPE_INDIVIDUAL;
-			 
-			if (recipient->status == E_GW_ITEM_STAT_ACCEPTED) {
-				const char *accept_level = e_gw_item_get_accept_level (item);
+				attendee->cn = g_strdup (recipient->display_name);
+				attendee->value = g_strconcat("MAILTO:", recipient->email, NULL);
+				if (recipient->type == E_GW_ITEM_RECIPIENT_TO)
+					attendee->role = ICAL_ROLE_REQPARTICIPANT;
+				else if (recipient->type == E_GW_ITEM_RECIPIENT_CC || recipient->type == E_GW_ITEM_RECIPIENT_BC)
+					attendee->role = ICAL_ROLE_OPTPARTICIPANT;
+				else 
+					attendee->role = ICAL_ROLE_NONE;
+				/* FIXME  needs a server fix on the interface 
+				 * for getting cutype and the status */
+				attendee->cutype = ICAL_CUTYPE_INDIVIDUAL;
 
-				if(accept_level && !strcmp (e_gw_item_get_accept_level (item),"Tentative"))
-					attendee->status = ICAL_PARTSTAT_TENTATIVE;
+				if (recipient->status == E_GW_ITEM_STAT_ACCEPTED) {
+					const char *accept_level = e_gw_item_get_accept_level (item);
+
+					if(accept_level && !strcmp (e_gw_item_get_accept_level (item),"Tentative"))
+						attendee->status = ICAL_PARTSTAT_TENTATIVE;
+					else
+						attendee->status = ICAL_PARTSTAT_ACCEPTED;
+				}
+				else if (recipient->status == E_GW_ITEM_STAT_DECLINED)
+					attendee->status = ICAL_PARTSTAT_DECLINED;
 				else
-					attendee->status = ICAL_PARTSTAT_ACCEPTED;
-			}
-			else if (recipient->status == E_GW_ITEM_STAT_DECLINED)
-				attendee->status = ICAL_PARTSTAT_DECLINED;
-			else
-				attendee->status = ICAL_PARTSTAT_NEEDSACTION;	
-				
-			
-			attendee_list = g_slist_append (attendee_list, attendee);				
-		}
+					attendee->status = ICAL_PARTSTAT_NEEDSACTION;	
 
-		e_cal_component_set_attendee_list (comp, attendee_list);
+
+				attendee_list = g_slist_append (attendee_list, attendee);				
+			}
+
+			e_cal_component_set_attendee_list (comp, attendee_list);
+		}
 	}
 
 	/* set organizer if it exists */
@@ -1111,9 +1221,9 @@ e_gw_item_to_cal_component (EGwItem *item, ECalBackendGroupwise *cbgw)
 			trigger.u.rel_duration = icaldurationtype_from_int (alarm_duration);
 			e_cal_component_alarm_set_trigger (alarm, trigger);
 			e_cal_component_add_alarm (comp, alarm);
-		}
 
-		
+		} else 
+			set_default_alarms (comp);
 
 		break;
 	case E_GW_ITEM_TYPE_TASK :
@@ -1174,6 +1284,8 @@ e_gw_item_to_cal_component (EGwItem *item, ECalBackendGroupwise *cbgw)
 			percent =0;
 		e_cal_component_set_percent (comp, &percent);
 
+		break;
+	case E_GW_ITEM_TYPE_NOTE:
 		break;
 	default :
 		return NULL;
@@ -1193,19 +1305,23 @@ e_gw_connection_send_appointment (ECalBackendGroupwise *cbgw, const char *contai
 	char *item_id = NULL;
 	const char *gw_id;
 	const char *recurrence_key = NULL;
-	gboolean need_to_get = FALSE;
+	gboolean need_to_get = FALSE, decline = FALSE;
+	ECalComponentVType type;
 
 	cnc = e_cal_backend_groupwise_get_connection (cbgw);
 	g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_INVALID_CONNECTION);
 	g_return_val_if_fail (E_IS_CAL_COMPONENT (comp), E_GW_CONNECTION_STATUS_INVALID_OBJECT);
 
 	e_cal_component_commit_sequence (comp);
+	type = e_cal_component_get_vtype (comp);
 
 	gw_id = e_cal_component_get_gw_id (comp);
 
-	switch  (e_cal_component_get_vtype (comp)) {
+	switch  (type) {
 
 		case E_CAL_COMPONENT_EVENT: 
+		case E_CAL_COMPONENT_TODO:
+		case E_CAL_COMPONENT_JOURNAL:
 			if (!g_str_has_suffix (gw_id, container)) {
 				item_id = g_strconcat (e_cal_component_get_gw_id (comp), GW_EVENT_TYPE_ID, container, NULL);
 				need_to_get = TRUE;
@@ -1214,15 +1330,6 @@ e_gw_connection_send_appointment (ECalBackendGroupwise *cbgw, const char *contai
 			else 
 				item_id = g_strdup (gw_id);
 			break;
-		case E_CAL_COMPONENT_TODO:
-			if (!g_str_has_suffix (gw_id, container)) {
-				item_id = g_strconcat (e_cal_component_get_gw_id (comp), GW_TODO_TYPE_ID, container, NULL);
-				need_to_get = TRUE;
-				
-			}
-			else
-				item_id = g_strdup (gw_id);
-			break;
 		default:
 			return E_GW_CONNECTION_STATUS_INVALID_OBJECT;
 	}
@@ -1242,6 +1349,24 @@ e_gw_connection_send_appointment (ECalBackendGroupwise *cbgw, const char *contai
 		g_object_unref (item);
 	}
 
+	if (type == E_CAL_COMPONENT_JOURNAL) {
+		icalcomponent *icalcomp = e_cal_component_get_icalcomponent (comp);
+		icalproperty *icalprop;
+
+		icalprop = icalcomponent_get_first_property (icalcomp, ICAL_X_PROPERTY);
+		while (icalprop) {
+			const char *x_name;
+
+			x_name = icalproperty_get_x_name (icalprop);
+			if (!strcmp (x_name, "X-GW-DECLINED")) {
+				decline = TRUE;
+				*pstatus = ICAL_PARTSTAT_DECLINED;
+				break;
+			}
+			icalprop = icalcomponent_get_next_property (icalcomp, ICAL_X_PROPERTY);
+		}
+	}
+
 	switch (method) {
 	case ICAL_METHOD_REQUEST:
 		/* get attendee here and add the list along. */
@@ -1324,6 +1449,12 @@ e_gw_connection_send_appointment (ECalBackendGroupwise *cbgw, const char *contai
 	case ICAL_METHOD_CANCEL:
 		status = e_gw_connection_retract_request (cnc, item_id, NULL, FALSE, FALSE);
 		break;
+	case ICAL_METHOD_PUBLISH:
+		if (decline)
+			status = e_gw_connection_decline_request (cnc, item_id, NULL, NULL);
+		else
+			status = e_gw_connection_accept_request (cnc, item_id, "Free",  NULL, NULL);
+		break;
 	default:
 		return E_GW_CONNECTION_STATUS_INVALID_OBJECT;
 	}
@@ -1786,9 +1917,11 @@ add_return_value (EGwSendOptionsReturnNotify track, ESource *source, char *notif
 	g_free (value), value = NULL;
 }
 
-void
-e_cal_backend_groupwise_store_settings (EGwSendOptions *opts, ECalBackendGroupwise *cbgw)
+gboolean
+e_cal_backend_groupwise_store_settings (GwSettings *hold)
 {
+	ECalBackendGroupwise *cbgw;
+	EGwSendOptions *opts;
 	EGwSendOptionsGeneral *gopts;
 	EGwSendOptionsStatusTracking *sopts;
 	icaltimetype tt;
@@ -1799,18 +1932,24 @@ e_cal_backend_groupwise_store_settings (EGwSendOptions *opts, ECalBackendGroupwi
 	const char *uid;
 	char *value;
 
+	cbgw = hold->cbgw;
+	opts = hold->opts;
 	source = e_cal_backend_get_source (E_CAL_BACKEND (cbgw));
 	kind = e_cal_backend_get_kind (E_CAL_BACKEND (cbgw)); 
 
+	/* TODO implement send options for Notes */
+	if (kind == ICAL_VJOURNAL_COMPONENT)
+		return FALSE;
+
 	gopts = e_gw_sendoptions_get_general_options (opts);
 	if (kind == ICAL_VEVENT_COMPONENT) {
-		sopts = e_gw_sendoptions_get_status_tracking_options (opts, "calendar");
 		source_list = e_source_list_new_for_gconf (gconf, "/apps/evolution/calendar/sources");
+		sopts = e_gw_sendoptions_get_status_tracking_options (opts, "calendar");
 	} else {
 		source_list = e_source_list_new_for_gconf (gconf, "/apps/evolution/tasks/sources");
 		sopts = e_gw_sendoptions_get_status_tracking_options (opts, "task");
-	}
-
+	} 
+	
 	uid = e_source_peek_uid (source);
 	source = e_source_list_peek_source_by_uid (source_list, uid);
 	if (gopts) {
@@ -1886,7 +2025,15 @@ e_cal_backend_groupwise_store_settings (EGwSendOptions *opts, ECalBackendGroupwi
 		add_return_value (sopts->completed, source, "return-complete"); 
 	}	
 
+	e_source_list_sync (source_list, NULL);
+
+	g_object_unref (hold->opts);
+	g_free (hold);
+
 	g_object_unref (gconf);
+	g_object_unref (source_list);
+
+	return FALSE;
 }
 
 gboolean
